package test;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
import software.amazon.awssdk.auth.credentials.StaticCredentialsProvider;
import software.amazon.awssdk.core.ResponseInputStream;
import software.amazon.awssdk.core.sync.RequestBody;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.s3.S3Client;
import software.amazon.awssdk.services.s3.model.Bucket;
import software.amazon.awssdk.services.s3.model.DeleteObjectRequest;
import software.amazon.awssdk.services.s3.model.GetObjectRequest;
import software.amazon.awssdk.services.s3.model.GetObjectResponse;
import software.amazon.awssdk.services.s3.model.HeadObjectRequest;
import software.amazon.awssdk.services.s3.model.ListBucketsRequest;
import software.amazon.awssdk.services.s3.model.ListBucketsResponse;
import software.amazon.awssdk.services.s3.model.NoSuchKeyException;
import software.amazon.awssdk.services.s3.model.PutObjectRequest;

public class TestS3 
{
	private final static String BUCKET_NAME = "bucket name";
	private final static String ACCESS_KEY_ID = "access key id";
	private final static String SECRET_ACCESS_KEY = "secret access key";
	
	public static void main(String [] args) throws Exception
	{
		// Create S3 Client
		Region region = Region.US_EAST_1;
        S3Client s3 = S3Client
        		.builder()
        		.region(region)
        		.credentialsProvider(StaticCredentialsProvider.create(AwsBasicCredentials.create(ACCESS_KEY_ID, SECRET_ACCESS_KEY)))
        		.build();
        
        // List buckets
        listBuckets(s3);

        // Put a file
        putFile(s3, "c:/qoppa/auto/test03.pdf");
        
        // Get a file
        getFile(s3, "test02.pdf", "c:/qoppa/tmp/test02.pdf");
        
        // Delete a file
        deleteFile(s3, "test03.pdf");
        
        // File exists
        exists(s3, "test03.pdf");
	}
	
	private static void listBuckets(S3Client s3)
	{
        ListBucketsRequest listBucketsRequest = ListBucketsRequest.builder().build();
        ListBucketsResponse listBucketsResponse = s3.listBuckets(listBucketsRequest);
        List<Bucket> bucketList = listBucketsResponse.buckets();
        for (Bucket b : bucketList)
        {
        	System.out.println(b.name());
        }
	}
	
	private static void putFile(S3Client s3, String filePath)
	{
		File putFile = new File(filePath);

        PutObjectRequest objectRequest = PutObjectRequest.builder()
                .bucket(BUCKET_NAME)
                .key(putFile.getName())
                .build();

        s3.putObject(objectRequest, RequestBody.fromFile(putFile));
	}
	
	private static void getFile(S3Client s3, String fileKey, String filePath) throws Exception
	{
		GetObjectRequest getRequest = GetObjectRequest.builder()
				.bucket(BUCKET_NAME)
				.key(fileKey)
				.build();
		
		ResponseInputStream<GetObjectResponse> respStream = s3.getObject(getRequest);

		FileOutputStream outstream = new FileOutputStream(filePath);
		copyStreams(respStream, outstream);
	}
	
	private static void deleteFile(S3Client s3, String fileKey)
	{
		DeleteObjectRequest delRequest = DeleteObjectRequest.builder()
				.bucket(BUCKET_NAME)
				.key(fileKey)
				.build();
		
		s3.deleteObject(delRequest);
	}
	
	public static void copyStreams (InputStream inStream, OutputStream outStream) throws IOException
	{
		byte [] buffer = new byte [1024];
		int bytesRead = inStream.read(buffer);
		while (bytesRead > 0)
		{
			outStream.write (buffer, 0, bytesRead);
			bytesRead = inStream.read (buffer);
		}
	}
	
	public static boolean exists(S3Client s3, String fileKey)
	{
		HeadObjectRequest headRequest = HeadObjectRequest.builder()
				.bucket(BUCKET_NAME)
				.key(fileKey)
				.build();
		
		try
		{
			s3.headObject(headRequest);
		}
		catch(NoSuchKeyException nske)
		{
			return false;
		}
		return true;
	}
}
